﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Net;
using System.Net.Http;
using System.Net.Http.Headers;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

using Newtonsoft.Json;

namespace TestProject
{
    /// <summary>
    /// 프로그램
    /// </summary>
    class Program
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Private

        #region 업로드하기 (비동기) - UploadAsync(url, mail, attachmentFileList)

        /// <summary>
        /// 업로드하기 (비동기)
        /// </summary>
        /// <param name="url">URL</param>
        /// <param name="mail">메일</param>
        /// <param name="attachmentFileList">첨부 파일 리스트</param>
        /// <returns>처리 결과 태스크</returns>
        private async static Task<string> UploadAsync(string url, Mail mail, List<AttachmentFile> attachmentFileList)
        {
            HttpRequestMessage requestMessage = new HttpRequestMessage(HttpMethod.Post, url);

            requestMessage.Headers.ExpectContinue = false;

            MultipartContent multipartContent = new MultipartContent("form-data", Guid.NewGuid().ToString());

            string mailContentJSON = JsonConvert.SerializeObject(mail, Formatting.Indented);

            StringContent mailContent = new StringContent(mailContentJSON, Encoding.UTF8);

            mailContent.Headers.ContentDisposition = new ContentDispositionHeaderValue("form-data") { Name = "mail" };
            mailContent.Headers.ContentType        = new MediaTypeHeaderValue("application/json");

            multipartContent.Add(mailContent);

            if(attachmentFileList != null)
            {
                foreach(AttachmentFile attachmentFile in attachmentFileList)
                {
                    FileStream fileStream = new FileStream(attachmentFile.FilePath, FileMode.Open, FileAccess.Read, FileShare.Read);

                    StreamContent fileContent = new StreamContent(fileStream);

                    fileContent.Headers.ContentDisposition = new ContentDispositionHeaderValue("form-data")
                    {
                        Name     = "attachments",
                        FileName = fileStream.Name
                    };

                    fileContent.Headers.ContentType = new MediaTypeHeaderValue("image/png");

                    multipartContent.Add(fileContent);
                }
            }

            requestMessage.Content = multipartContent;

            HttpClient httpClient = new HttpClient();

            try
            {
                HttpResponseMessage responseMessage = await httpClient.SendAsync
                (
                    requestMessage,
                    HttpCompletionOption.ResponseContentRead,
                    CancellationToken.None
                );

                HttpContent responseContent = responseMessage.Content;

                string responseResult = await responseContent.ReadAsStringAsync();

                return responseResult;
            }
            catch(Exception exception)
            {
                Console.WriteLine(exception.ToString());

                return null;
            }
        }

        #endregion
        #region 프로그램 시작하기 - Main()

        /// <summary>
        /// 프로그램 시작하기
        /// </summary>
        private static void Main()
        {
            ServicePointManager.SecurityProtocol = SecurityProtocolType.Tls12;

            string url = "https://dscore.com:8000/api/v1/common/mail/outlook/send";

            Mail mail = new Mail();

            mail.UserID       = "gildong.hong";
            mail.MailID       = "51EE6D39-009E-4EB8-A7DC-896892195FE3";
            mail.Subject      = "테스트";
            mail.Content      = "테스트 메일 입니다.";
            mail.ContentType  = ContentType.Text;
            mail.DocumentType = DocumentType.Personal;
            mail.SendTime     = DateTime.UtcNow.ToString("yyyy-MM-ddTHH:mm:ss.sssZ");
            mail.Sender       = new Sender { Name = "홍길동", EMailAddress = "hong@daum.net" };

            mail.RecipientList.Add(new Recipient { Name = "김철수", EMailAddress = "kim@daum.net", RecipientType = RecipientType.TO });

            List<AttachmentFile> attachmentFileList = new List<AttachmentFile>();

            attachmentFileList.Add(new AttachmentFile { FilePath = "d:\\sample.png" });

            string result = UploadAsync(url, mail, attachmentFileList).Result;

            Console.WriteLine(result);

            Console.ReadKey(false);
        }

        #endregion
    }
}